////////////////////////////////////////////////////////////
// RegUtil class interface
//   Demo class for ObjectOrientedProgramming
//   03/05/04 - Burns
#include <string>
#include <algorithm>
#include <windows.h>
using namespace std;

const int MAX_STRING_LEN = 256;

class RegUtil
{
private:
  ////////////////////////////////////////////////////////////
  // parseRegKey - split reg key into base hive and sub key
  //  IN szRegKey     - Key within Registry
  //                    ex: HKEY_CURRENT_USER\Software\Testing
  //  OUT regSubKey   - sub key contained in szRegKey
  //  RETURN          - HKEY to base hive
  static HKEY parseRegKey( const char* szRegKey, string& regSubKey )
  {
    string regKey = szRegKey;
    long splitAt;
    if ( ( splitAt = regKey.find_first_of( '\\' ) ) != string::npos )
    {
      regSubKey = regKey.substr( splitAt + 1, regKey.length() - splitAt - 1 );
      regKey = regKey.substr( 0, splitAt );
      
      // convert to uppercase since STL string compare is case sensitive
      transform( regKey.begin(), regKey.end(), regKey.begin(), toupper );
      
      // which root hive key is it?
      if ( regKey == "HKEY_CURRENT_USER" )
      {
        return HKEY_CURRENT_USER;
      }
      else if ( regKey == "HKEY_LOCAL_MACHINE" )
      {
        return HKEY_LOCAL_MACHINE;
      }
      else if ( regKey == "HKEY_CLASSES_ROOT" )
      {
        return HKEY_CLASSES_ROOT;
      }
      else if ( regKey == "HKEY_USERS" )
      {
        return HKEY_USERS;
      }
      else
      {
        // could not match key
        return NULL;
      }
    }
    else
    {
      return NULL;
    }
  }


public:
  ////////////////////////////////////////////////////////////
  // getRegValue
  //  IN regKey       - Key within Registry
  //                    ex: HKEY_CURRENT_USER\Software\Testing
  //  IN regValueName - Name of value in key
  //  OUT regValueString - String read from value
  //  RETURN          - true if successful
  static bool getRegValue( const char* regKey, const char* regValueName, char* regValueString )
  {
    string regSubKey;
    HKEY hBaseKey = parseRegKey( regKey, regSubKey );
    HKEY hKey;

    // try to open the key for reading
    if ( RegOpenKeyEx( hBaseKey, regSubKey.c_str(), 0, 
                       KEY_READ, &hKey ) == ERROR_SUCCESS )
    {
      // we openned it
      DWORD dataType;
      DWORD dataSize = MAX_STRING_LEN;
      byte  data[ MAX_STRING_LEN ];
      if ( RegQueryValueEx( hKey, regValueName, 0, &dataType, data,
                            &dataSize ) == ERROR_SUCCESS )
      {
        if ( dataType == REG_DWORD )
        {
          long dataNum;
          dataNum = (DWORD)data[0];
          itoa( dataNum, regValueString, 10 );
        }
        else
        {
          strncpy( regValueString, (char*)data, dataSize );
        }
        RegCloseKey( hKey );
        return true;
      }
      RegCloseKey( hKey );
    }
    return false;
  }

  ////////////////////////////////////////////////////////////
  // getRegValue
  //  IN regKey       - Key within Registry
  //                    ex: HKEY_CURRENT_USER\Software\Testing
  //  IN regValueName - Name of value in key
  //  OUT regIntValue - Integer read from value
  //  RETURN          - true if successful
  static bool getRegValue( const char* regKey, const char* regValueName, int regIntValue )
  {
    string regSubKey;
    HKEY hBaseKey = parseRegKey( regKey, regSubKey );
    HKEY hKey;

    // try to open the key for reading
    if ( RegOpenKeyEx( hBaseKey, regSubKey.c_str(), 0, 
                       KEY_READ, &hKey ) == ERROR_SUCCESS )
    {
      // we openned it
      DWORD dataType;
      DWORD dataSize = MAX_STRING_LEN;
      byte  data[ MAX_STRING_LEN ];
      if ( RegQueryValueEx( hKey, regValueName, 0, &dataType, data,
                            &dataSize ) == ERROR_SUCCESS )
      {
        if ( dataType == REG_DWORD )
        {
          regIntValue = (DWORD)data[0];
        }
        else
        {
          char temp[ MAX_STRING_LEN ];
          strncpy( temp, (char*)data, dataSize );
          regIntValue = atoi( temp );
        }
        RegCloseKey( hKey );
        return true;
      }
      RegCloseKey( hKey );
    }
    return false;
  }

  ////////////////////////////////////////////////////////////
  // setRegValue
  //  IN regKey       - Key within Registry
  //                    ex: HKEY_CURRENT_USER\Software\Testing
  //  IN regValueName - Name of value in key
  //  In regIntValue  - Integer to set value to
  //  RETURN          - true if successful
  static bool setRegValue( const char* regKey, const char* regValueName, const int regIntValue )
  {
    string regSubKey;
    HKEY hBaseKey = parseRegKey( regKey, regSubKey );
    HKEY hKey;

    // try to open the key for writing
    if ( RegCreateKeyEx( hBaseKey, regSubKey.c_str(), 0, NULL, 
                         REG_OPTION_NON_VOLATILE, KEY_WRITE,
                         NULL, &hKey, NULL ) == ERROR_SUCCESS )
    {
      // we openned it, now store our value       
      if ( RegSetValueEx( hKey, regValueName, 0, REG_DWORD, (byte*)&regIntValue,
                          sizeof( DWORD ) ) == ERROR_SUCCESS )
      {
        RegCloseKey( hKey );
        return true;
      }
      RegCloseKey( hKey );
    }
    return false;
  }

  ////////////////////////////////////////////////////////////
  // setRegValue
  //  IN regKey       - Key within Registry
  //                    ex: HKEY_CURRENT_USER\Software\Testing
  //  IN regValueName - Name of value in key
  //  In regValueString  - String to set value to
  //  RETURN          - true if successful
  static bool setRegValue( const char* regKey, const char* regValueName, const char* regValueString )
  {
    string regSubKey;
    HKEY hBaseKey = parseRegKey( regKey, regSubKey );
    HKEY hKey;

    // try to open the key for writing
    if ( RegCreateKeyEx( hBaseKey, regSubKey.c_str(), 0, NULL, 
                         REG_OPTION_NON_VOLATILE, KEY_WRITE,
                         NULL, &hKey, NULL ) == ERROR_SUCCESS )
    {
      // we openned it, now store our value       
      if ( RegSetValueEx( hKey, regValueName, 0, REG_SZ, (byte*)regValueString,
                          strlen( regValueString ) ) == ERROR_SUCCESS )
      {
        return true;
      }
      RegCloseKey( hKey );
    }
    return false;
  }

  ////////////////////////////////////////////////////////////
  // deleteRegValue
  //  IN regKey       - Key within Registry
  //                    ex: HKEY_CURRENT_USER\Software\Testing
  //  IN regValueName - Name of value in key to delete
  //  RETURN          - true if successful
  static bool deleteRegValue( const char* regKey, const char* regValueName )
  {
    string regSubKey;
    HKEY hBaseKey = parseRegKey( regKey, regSubKey );
    HKEY hKey;

    // try to open the key for writing
    if ( RegOpenKeyEx( hBaseKey, regSubKey.c_str(), 0, 
                       KEY_SET_VALUE, &hKey ) == ERROR_SUCCESS )
    {
      if ( RegDeleteValue( hKey, regValueName ) == ERROR_SUCCESS )
      {
        RegCloseKey( hKey );
        return true;
      }
      RegCloseKey( hKey );
    }
    return false;
  }

  ////////////////////////////////////////////////////////////
  // deleteRegValue
  //  IN regKey       - Key within Registry to delete
  //                    ex: HKEY_CURRENT_USER\Software\Testing
  //  RETURN          - true if successful
  static bool deleteRegKey( const char* regKey )
  {
    string regSubKey;
    HKEY hBaseKey = parseRegKey( regKey, regSubKey );
    
    // delete the key
    if ( RegDeleteKey( hBaseKey, regSubKey.c_str() ) == ERROR_SUCCESS )
    {
      return true;
    }
    return false;
  }
  
};
